@@ -10,14 +10,14 @@
 *
 * $Project: /Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2003/04/12 04:44:11 +0200 $
-* $Revision: 4 $
-* $Snapshot: /Convert-Binary-C/0.49 $
+* $Date: 2004/03/22 20:37:55 +0100 $
+* $Revision: 6 $
+* $Snapshot: /Convert-Binary-C/0.50 $
 * $Source: /ctlib/byteorder.h $
 *
 ********************************************************************************
 *
-* Copyright (c) 2002-2003 Marcus Holland-Moritz. All rights reserved.
+* Copyright (c) 2002-2004 Marcus Holland-Moritz. All rights reserved.
 * This program is free software; you can redistribute it and/or modify
 * it under the same terms as Perl itself.
 *
@@ -57,6 +57,9 @@ typedef struct {
 
 /*===== FUNCTION PROTOTYPES ==================================================*/
 
+#define string_is_integer CTlib_string_is_integer
+int string_is_integer(const char *pStr);
+
 #define fetch_integer CTlib_fetch_integer
 void fetch_integer( unsigned size, unsigned sign, const void *src,
                     const ArchSpecs *pAS, IntValue *pIV );
