@@ -10,27 +10,29 @@
 *
 * $Project: /Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2003/11/21 13:50:37 +0100 $
-* $Revision: 9 $
-* $Snapshot: /Convert-Binary-C/0.49 $
+* $Date: 2004/03/22 20:37:56 +0100 $
+* $Revision: 11 $
+* $Snapshot: /Convert-Binary-C/0.50 $
 * $Source: /ctlib/cterror.h $
 *
 ********************************************************************************
 *
-* Copyright (c) 2002-2003 Marcus Holland-Moritz. All rights reserved.
+* Copyright (c) 2002-2004 Marcus Holland-Moritz. All rights reserved.
 * This program is free software; you can redistribute it and/or modify
 * it under the same terms as Perl itself.
 *
 *******************************************************************************/
 
-#ifndef _CTLIB_CPPERR_H
-#define _CTLIB_CPPERR_H
+#ifndef _CTLIB_CTERROR_H
+#define _CTLIB_CTERROR_H
 
 /*===== GLOBAL INCLUDES ======================================================*/
 
 /*===== LOCAL INCLUDES =======================================================*/
 
 #include "ctparse.h"
+#include "cppreent.h"
+
 
 /*===== DEFINES ==============================================================*/
 
@@ -56,6 +58,11 @@ typedef struct {
   char *string;
 } CTLibError;
 
+#ifdef UCPP_REENTRANT
+struct CPP;
+#endif
+
+
 /*===== FUNCTION PROTOTYPES ==================================================*/
 
 #define set_print_functions CTlib_set_print_functions
@@ -73,4 +80,17 @@ void push_warning( CParseInfo *pCPI, const char *fmt, ... );
 #define fatal_error CTlib_fatal_error
 void fatal_error( const char *fmt, ... );
 
+#ifdef UCPP_REENTRANT
+
+#define my_ucpp_ouch CTlib_my_ucpp_ouch
+void my_ucpp_ouch( pUCPP_ char *fmt, ... );
+
+#define my_ucpp_error CTlib_my_ucpp_error
+void my_ucpp_error( pUCPP_ long line, char *fmt, ... );
+
+#define my_ucpp_warning CTlib_my_ucpp_warning
+void my_ucpp_warning( pUCPP_ long line, char *fmt, ... );
+
+#endif
+
 #endif
